/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.business;

import fr.paris.lutece.plugins.stock.service.StockPlugin;
import fr.paris.lutece.portal.service.jpa.JPALuteceDAO;

import java.lang.reflect.ParameterizedType;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;


/**
 * This class provides Data Access methods for Person objects
 * @param <K> the primary key
 * @param <T> the ticket category
 */
public class AbstractTicketCategoryDAO<K, T extends TicketCategory> extends JPALuteceDAO<K, T>
    implements ITicketCategoryDAO<K, T>
{
    private Class<T> _entityClass;

    /**
     * Constructor
     */
    public AbstractTicketCategoryDAO(  )
    {
        _entityClass = (Class<T>) ( (ParameterizedType) getClass(  ).getGenericSuperclass(  ) ).getActualTypeArguments(  )[1];
    }

    /**
     * Get the product P class
     * @return the class
     */
    public Class<T> getProductClass(  )
    {
        return _entityClass;
    }

    /**
    *
    *{@inheritDoc}
    */
    @Override
    public String getPluginName(  )
    {
        return StockPlugin.PLUGIN_NAME;
    }

    /**
     * Find ticket by id with children fetched
     * @param idCategory the ticket category id
     * @return the ticket category
     */
    public T findByIdWithChildren( K idCategory )
    {
        T ticketCategory = findById( idCategory );
        //in order to load the children
        ticketCategory.getChildrenList(  ).size(  );

        return ticketCategory;
    }

    /**
     * Find ticket by id with parent fetched
     * @param idCategory the ticket category id
     * @return the ticket category
     */
    public T findByIdWithParent( K idCategory )
    {
        T ticketCategory = findById( idCategory );

        //in order to load the parent
        Category parent = ticketCategory.getParent(  );

        if ( parent != null )
        {
            parent.getChildrenList(  ).size(  );
        }

        return ticketCategory;
    }

    /**
     * Find all ticket category without parents, with children fetched
     * @return the ticket category list
     */
    public List<T> selectAllFirstLevelWithChildren(  )
    {
        EntityManager em = getEM(  );
        CriteriaBuilder cb = em.getCriteriaBuilder(  );

        CriteriaQuery<T> cq = cb.createQuery( getProductClass(  ) );

        Root<T> root = cq.from( getProductClass(  ) );
        root.fetch( T.ATTRIBUTE_CHILDREN_LIST, JoinType.LEFT );

        Predicate condition = cb.isNull( root.get( TicketCategory.ATTRIBUTE_PARENT ) );
        cq.where( condition );

        TypedQuery<T> query = em.createQuery( cq );

        return query.getResultList(  );
    }
}
