/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.business;

import fr.paris.lutece.plugins.stock.service.GlobalProviderService;
import fr.paris.lutece.plugins.stock.utils.jpa.StockJPAUtils;

import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.TableGenerator;
import javax.persistence.Transient;


/**
 *
 * Provider
 *
 */
@Entity
@Inheritance( strategy = InheritanceType.JOINED )
@Table( name = "stock_provider" )
public class Provider
{
    public static final String RESOURCE_TYPE = "PROVIDER_REFERENCE_TYPE";

    /** Sequence name */
    private static final String JPA_SEQUENCE_NAME = "stock_provider_sequence";

    /** Unique value */
    private static final String JPA_COLUNM_NAME = "stock_provider_id";
    private int _idProvider;
    private String _strName;
    private String _strAddress;
    private String _strContactName;
    private String _strPhoneNumber;
    private String _strMail;
    private String _strComment;
    private List<Category> _listProducts;

    /**
     * Constructor
     */
    public Provider(  )
    {
    }

    /**
     * Construct a new provider from an other
     * @param provider a provider object to copy
     */
    public Provider( Provider provider )
    {
        copy( provider );
    }

    /**
     * Copy values from a provider object to this
     * @param provider a provider object
     */
    private void copy( Provider provider )
    {
        setIdProvider( provider.getIdProvider(  ) );
        setName( provider.getName(  ) );
        setAddress( provider.getAddress(  ) );
        setContactName( provider.getContactName(  ) );
        setPhoneNumber( provider.getPhoneNumber(  ) );
        setMail( provider.getMail(  ) );
        setComment( provider.getComment(  ) );
        setProducts( provider.getProducts(  ) );
    }

    /**
     * Return the provider id
     * @return the provider id
     */
    @TableGenerator( table = StockJPAUtils.SEQUENCE_TABLE_NAME, name = JPA_SEQUENCE_NAME, pkColumnValue = JPA_COLUNM_NAME, allocationSize = 1 )
    @Id
    @GeneratedValue( strategy = GenerationType.TABLE, generator = JPA_SEQUENCE_NAME )
    @Column( name = "id_provider" )
    public int getIdProvider(  )
    {
        return _idProvider;
    }

    /**
     * Set the provider id
     * @param idProvider the provider id
     */
    public void setIdProvider( int idProvider )
    {
        _idProvider = idProvider;
    }

    /**
     * Get the phone number to contact the provider
     * @return the phone number
     */
    @Column( name = "phone_number" )
    public String getPhoneNumber(  )
    {
        return _strPhoneNumber;
    }

    /**
     * Set the phone number
     * @param strPhoneNumber the phone number
     */
    public void setPhoneNumber( String strPhoneNumber )
    {
        _strPhoneNumber = strPhoneNumber;
    }

    /**
     * Get the provider address
     * @return the provider address
     */
    @Column( name = "address" )
    public String getAddress(  )
    {
        return _strAddress;
    }

    /**
     * Set the provider address
     * @param address address
     */
    public void setAddress( String address )
    {
        _strAddress = address;
    }

    /**
     * Get the provider mail
     * @return the mail
     */
    @Column( name = "mail" )
    public String getMail(  )
    {
        return _strMail;
    }

    /**
     * Set the provider mail
     * @param mail the mail
     */
    public void setMail( String mail )
    {
        _strMail = mail;
    }

    /**
     * Get the provider name
     * @return the provider name
     */
    @Column( name = "name" )
    public String getName(  )
    {
        return _strName;
    }

    /**
     * Set the provider name
     * @param name the provider name
     */
    public void setName( String name )
    {
        _strName = name;
    }

    /**
     * Get the product list
     * @return the product list
     */
    @OneToMany( cascade = CascadeType.ALL, fetch = FetchType.LAZY )
    public List<Category> getProducts(  )
    {
        return _listProducts;
    }

    /**
     * Set the product list
     * @param products the product list
     */
    public void setProducts( List<Category> products )
    {
        _listProducts = products;
    }

    /**
     * Get the contact name
     * @return the contact name
     */
    public String getContactName(  )
    {
        return _strContactName;
    }

    /**
     * Set the contact name
     * @param contactName the name of the contact
     */
    public void setContactName( String contactName )
    {
        this._strContactName = contactName;
    }

    /**
     * Get the comment
     * @return the comment
     */
    public String getComment(  )
    {
        return _strComment;
    }

    /**
     * Set the comment
     * @param comment the comment
     */
    public void setComment( String comment )
    {
        this._strComment = comment;
    }

    /**
     * Get the object definition for provider class
     * @return the object definition
     */
    @Transient
    public ObjectDefinition getObjectDefinition(  )
    {
        return GlobalProviderService.getInstance(  ).getObjectDefinition( this.getClass(  ).getName(  ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode(  )
    {
        final int prime = 31;
        int result = 1;
        result = ( prime * result ) + _idProvider;

        return result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals( Object obj )
    {
        if ( this == obj )
        {
            return true;
        }

        if ( obj == null )
        {
            return false;
        }

        if ( getClass(  ) != obj.getClass(  ) )
        {
            return false;
        }

        Provider other = (Provider) obj;

        if ( _idProvider != other._idProvider )
        {
            return false;
        }

        return true;
    }
}
