/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.Category;
import fr.paris.lutece.plugins.stock.business.CategoryFilter;
import fr.paris.lutece.plugins.stock.business.ICategoryHome;
import fr.paris.lutece.plugins.stock.business.IProviderHome;
import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Provider;
import fr.paris.lutece.plugins.stock.utils.StockUtils;
import fr.paris.lutece.plugins.stock.web.StockJspBean;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.rbac.RBACService;

import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * CategoryService
 *
 */
@Transactional( readOnly = true )
public final class CategoryService implements ICategoryService
{
    private static final String TEMPLATE_CREATE_CATEGORY = "/admin/plugins/stock/create_category.html";
    private static final String TEMPLATE_MODIFY_CATEGORY = "/admin/plugins/stock/modify_category.html";
    private static final String MARK_CATEGORY = "category";
    private static final String CATEGORY_NAME_KEY = "stock.create_category.type.basic.name";

    //JSP
    private static final String CATEGORY_CREATION_JSP = "jsp/admin/plugins/stock/CreateCategory.jsp";
    private static final String CATEGORY_MODIFICATION_JSP = "jsp/admin/plugins/stock/ModifyCategory.jsp";
    private static final String CATEGORY_DELETION_JSP = "jsp/admin/plugins/stock/DeleteCategory.jsp";
    private static final String CATEGORY_DO_DELETE_JSP = "jsp/admin/plugins/stock/DoDeleteCategory.jsp";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_CREATE_BASIC_CATEGORY = "stock.create_category.error.basiccategory.notAuthorized";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_DELETE_BASIC_CATEGORY = "stock.delete_category.error.basiccategory.notAuthorized";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_MODIFY_BASIC_CATEGORY = "stock.modify_category.error.basiccategory.notAuthorized";
    private static CategoryService _instance = new CategoryService(  );
    private static ObjectDefinition _objectDefinition;
    private ICategoryHome _homeCategory;
    private IProviderHome _homeProvider;

    /**
     * Private constructor
     */
    private CategoryService(  )
    {
    }

    /**
     * Get the {@link IProviderHome} object
     * @return the {@link IProviderHome} object
     */
    public IProviderHome getHomeProvider(  )
    {
        return _homeProvider;
    }

    /**
     * Set the {@link IProviderHome} object
     * @param homeProvider the new provider home
     */
    public void setHomeProvider( IProviderHome homeProvider )
    {
        _homeProvider = homeProvider;
    }

    /**
     * Get the {@link ICategoryHome} object
     * @return the {@link ICategoryHome} object
     */
    public ICategoryHome getHomeCategory(  )
    {
        return _homeCategory;
    }

    /**
     * Set the {@link ICategoryHome} object
     * @param homeCategory the new category home
     */
    public void setHomeCategory( ICategoryHome homeCategory )
    {
        _homeCategory = homeCategory;
    }

    /**
     * Get an instance of {@link CategoryService}
     * @return An instance of {@link CategoryService}
     */
    public static CategoryService getInstance(  )
    {
        return _instance;
    }

    /**
     * {@inheritDoc}
     */
    public String getCreateTemplate(  )
    {
        return TEMPLATE_CREATE_CATEGORY;
    }

    /**
     * {@inheritDoc}
     */
    public String getModifyTemplate(  )
    {
        return TEMPLATE_MODIFY_CATEGORY;
    }

    /**
     * {@inheritDoc}
     */
    public String getDeleteJsp(  )
    {
        return CATEGORY_DO_DELETE_JSP;
    }

    /**
     * {@inheritDoc}
     */
    public String getCategoryClass(  )
    {
        return Category.class.getName(  );
    }

    /**
     * {@inheritDoc}
     */
    public void init(  )
    {
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public String doCreateCategory( Category category, HttpServletRequest request )
    {
        _homeCategory.create( category );

        Category parent = category.getParent(  );

        if ( parent != null )
        {
            Category trueParent = _homeCategory.findByPrimaryKey( parent.getIdCategory(  ) );

            if ( trueParent != null )
            {
                trueParent.getChildrenList(  ).add( category );
                _homeCategory.update( trueParent );
            }
            else
            {
                return StockUtils.MESSAGE_ERROR_OCCUR;
            }
        }

        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public String doModifyCategory( Category category, HttpServletRequest request )
    {
        _homeCategory.update( category );

        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public void doDeleteCategory( int nIdCategory )
    {
        Category categoryToDelete = _homeCategory.findByPrimaryKey( nIdCategory );

        // Remove the category from the provider's categories
        Provider provider = _homeProvider.findByPrimaryKey( categoryToDelete.getProvider(  ).getIdProvider(  ) );
        provider.getProducts(  ).remove( categoryToDelete );
        _homeProvider.update( provider );
        // Remove the category
        _homeCategory.remove( nIdCategory );
    }

    /**
     * {@inheritDoc}
     */
    public void initModelForModification( int nIdCategory, Map<String, Object> model )
    {
        Category category = getCategory( nIdCategory );
        model.put( MARK_CATEGORY, category );
    }

    /**
     * {@inheritDoc}
     */
    public void initModelForCreation( Map<String, Object> model )
    {
        //nothing to do
    }

    /**
     * {@inheritDoc}
     */
    public Category getCategory( int nIdCategory )
    {
        return _homeCategory.findByPrimaryKey( nIdCategory );
    }

    /**
     *
     *{@inheritDoc}
     */
    public Category getCategoryWithParent( int nIdCategory )
    {
        return _homeCategory.findByIdWithParent( nIdCategory );
    }

    /**
     * {@inheritDoc}
     */
    public ObjectDefinition getObjectDefinition(  )
    {
        if ( _objectDefinition == null )
        {
            _objectDefinition = new ObjectDefinition( CATEGORY_NAME_KEY, CATEGORY_CREATION_JSP,
                    CATEGORY_MODIFICATION_JSP, CATEGORY_DELETION_JSP );
        }

        return _objectDefinition;
    }

    /**
     * {@inheritDoc}
     */
    public Category getCategoryWithChildren( Integer idCategory )
    {
        return _homeCategory.findByPrimarykeyWithChildren( idCategory );
    }

    /**
     * {@inheritDoc}
     */
    public List<Category> findByFilter( CategoryFilter filter )
    {
        return _homeCategory.findByFilter( filter );
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToCreate( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_CREATE_CATEGORY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToModify( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_MODIFY_CATEGORY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToDelete( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_DELETE_CATEGORY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public String getMessageNotAuthorizedForCreate(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_CREATE_BASIC_CATEGORY;
    }

    /**
     * {@inheritDoc}
     */
    public String getMessageNotAuthorizedForDelete(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_DELETE_BASIC_CATEGORY;
    }

    /**
     * {@inheritDoc}
     */
    public String getMessageNotAuthorizedForModification(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_MODIFY_BASIC_CATEGORY;
    }
}
