/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote categorys derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.Category;
import fr.paris.lutece.plugins.stock.business.CategoryFilter;
import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.portal.business.user.AdminUser;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * This class provides generics methods for {@link ICategoryService} implementations
 */
public final class GlobalCategoryService
{
    private static GlobalCategoryService _instance;
    private Map<String, ICategoryService> _services;

    /**
     * Private constructor
     */
    private GlobalCategoryService(  )
    {
        _services = new HashMap<String, ICategoryService>(  );
    }

    /**
     * Get the instance of this class
     * @return the instance of this class
     */
    public static GlobalCategoryService getInstance(  )
    {
        if ( _instance == null )
        {
            _instance = new GlobalCategoryService(  );
        }

        return _instance;
    }

    /**
     * Register a {@link ICategoryService} service
     * @param service the service to be registered
     */
    public void register( ICategoryService service )
    {
        _services.put( service.getCategoryClass(  ), service );
    }

    /**
     * Get the creation template of a service
     * @param strCategoryClassName The class name of the category entity to create
     * @return the relative path of the creation template
     */
    public String getCreateTemplate( String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getCreateTemplate(  );
    }

    /**
     * Get the modification template of a service
     * @param strCategoryClassName The class name of the category entity to modify
     * @return the relative path of the modification template
     */
    public String getModifyTemplate( String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getModifyTemplate(  );
    }

    /**
     * Get the deletion jsp of a service
     * @param strCategoryClassName The class name of the category entity to modify
     * @return the relative path of the deletion jsp
     */
    public String getDeleteJsp( String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getDeleteJsp(  );
    }

    /**
     * Creates a category
     * @param strCategoryClassName The class name of the category entity to create
     * @param category The category to create in database
     * @param request The HTTP request
     * @return error if occurs
     */
    public String doCreateCategory( String strCategoryClassName, Category category, HttpServletRequest request )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.doCreateCategory( category, request );
    }

    /**
     * Modifies a category
     * @param strCategoryClassName The class name of the category entity to modify
     * @param category The category to modify in database
     * @param request The HTTP request
     * @return error if occurs
     */
    public String doModifyCategory( String strCategoryClassName, Category category, HttpServletRequest request )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.doModifyCategory( category, request );
    }

    /**
     * Deletes a category
     * @param strCategoryClassName The class name of the category entity to delete
     * @param nIdCategory The category id to delete in database
     */
    public void doDeleteCategory( String strCategoryClassName, int nIdCategory )
    {
        ICategoryService service = _services.get( strCategoryClassName );
        service.doDeleteCategory( nIdCategory );
    }

    /**
     * Initaliaze the model in parameter for a modification
     * @param strCategoryClassName The class name of the category entity to delete
     * @param nIdCategory The category id to modify
     * @param model the model which will be passed in the modification template
     */
    public void initModelForModification( String strCategoryClassName, int nIdCategory, Map<String, Object> model )
    {
        ICategoryService service = _services.get( strCategoryClassName );
        service.initModelForModification( nIdCategory, model );
    }

    /**
     * Initaliaze the model in parameter for a creation
     * @param strCategoryClassName The class name of the category entity to create
     * @param model the model which will be passed in the modification template
     */
    public void initModelForCreation( String strCategoryClassName, Map<String, Object> model )
    {
        ICategoryService service = _services.get( strCategoryClassName );
        service.initModelForCreation( model );
    }

    /**
     * Get the category with the given id
     * @param strCategoryClassName The class name of the category entity to delete
     * @param nIdCategory The category id
     * @return The category with the given id
     */
    public Category getCategory( String strCategoryClassName, int nIdCategory )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getCategory( nIdCategory );
    }

    /**
     * Get all the registered services
     * @return The list of all the registered services
     */
    public List<ICategoryService> getAllRegisteredServices(  )
    {
        return new ArrayList<ICategoryService>( _services.values(  ) );
    }

    /**
     * Get the {@link ObjectDefinition} objects of all the registered services
     * @return The {@link ObjectDefinition} objects of all the registered services
     */
    public List<ObjectDefinition> getAllRegisteredObjectDefinition(  )
    {
        Collection<ICategoryService> listServices = _services.values(  );
        List<ObjectDefinition> listAllObjectDefinitions = new ArrayList<ObjectDefinition>(  );

        for ( ICategoryService service : listServices )
        {
            listAllObjectDefinitions.add( service.getObjectDefinition(  ) );
        }

        return listAllObjectDefinitions;
    }

    /**
     * Finds a category and fetch parent
     * @param strCategoryClassName The class name of the category entity
     * @param nIdCategory the id of the searched category
     * @return the category
     */
    public Category getCategoryWithParent( String strCategoryClassName, int nIdCategory )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getCategoryWithParent( nIdCategory );
    }

    /**
     * Finds the {@link ObjectDefinition} objects of the given category entity name
     * @param strCategoryClassName The class name of the category entity
     * @return the {@link ObjectDefinition} objects of the given category class name
     */
    public ObjectDefinition getObjectDefinition( String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getObjectDefinition(  );
    }

    /**
     * Finds a category and fetch children
     * @param strCategoryClassName The class name of the category entity
     * @param nIdCategory the id of the searched category
     * @return the category
     */
    public Category getCategoryWithChildren( String strCategoryClassName, int nIdCategory )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getCategoryWithChildren( nIdCategory );
    }

    /**
     * Finds all categories which match the given filter
     * @param strCategoryClassName The class name of the category entity
     * @param filter The filter
     * @return A list of categories which match the given filter
     */
    public List<Category> findByFilter( String strCategoryClassName, CategoryFilter filter )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.findByFilter( filter );
    }

    /**
     * Checks if a user is authorized to create the given category entity name
     * @param user the user
     * @param strCategoryClassName The class name of the category entity
     * @return true if the given user is authorized to create the given category entity. False otherwise.
     */
    public boolean isAuthorizedToCreate( AdminUser user, String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.isAuthorizedToCreate( user );
    }

    /**
     * Checks if a user is authorized to modify the given category entity name
     * @param user the user
     * @param strCategoryClassName The class name of the category entity
     * @return true if the given user is authorized to modify the given category entity. False otherwise.
     */
    public boolean isAuthorizedToModify( AdminUser user, String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.isAuthorizedToModify( user );
    }

    /**
     * Checks if a user is authorized to delete the given category entity name
     * @param user the user
     * @param strCategoryClassName The class name of the category entity
     * @return true if the given user is authorized to delete the given category entity. False otherwise.
     */
    public boolean isAuthorizedToDelete( AdminUser user, String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.isAuthorizedToDelete( user );
    }

    /**
     * Get the message when a user is not authorized to create a category
     * @param strCategoryClassName he class name of the category entity
     * @return The message
     */
    public String getMessageNotAuthorizedForCreate( String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getMessageNotAuthorizedForCreate(  );
    }

    /**
     * Get the message when a user is not authorized to modify a category
     * @param strCategoryClassName he class name of the category entity
     * @return The message
     */
    public String getMessageNotAuthorizedForModification( String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getMessageNotAuthorizedForModification(  );
    }

    /**
     * Get the message when a user is not authorized to delete a category
     * @param strCategoryClassName he class name of the category entity
     * @return The message
     */
    public String getMessageNotAuthorizedForDelete( String strCategoryClassName )
    {
        ICategoryService service = _services.get( strCategoryClassName );

        return service.getMessageNotAuthorizedForDelete(  );
    }
}
