/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Provider;
import fr.paris.lutece.portal.business.user.AdminUser;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * This class provides generics methods for {@link IProviderService} implementations
 */
public final class GlobalProviderService
{
    private static GlobalProviderService _singleton = new GlobalProviderService(  );
    private Map<String, IProviderService> _services;

    /**
     * Private constructor
     */
    private GlobalProviderService(  )
    {
        _services = new HashMap<String, IProviderService>(  );
    }

    /**
     * Get the instance of this class
     * @return the instance of this class
     */
    public static GlobalProviderService getInstance(  )
    {
        return _singleton;
    }

    /**
     * Register a {@link IProviderService} service
     * @param service the service to be registered
     */
    public void register( IProviderService service )
    {
        _services.put( service.getProviderClass(  ), service );
    }

    /**
     * Get the creation template of a service
     * @param strProviderClassName The class name of the provider entity to create
     * @return the relative path of the creation template
     */
    public String getCreateTemplate( String strProviderClassName )
    {
        IProviderService service = _services.get( strProviderClassName );

        return service.getCreateTemplate(  );
    }

    /**
     * Get the modification template of a service
     * @param strProviderClassName The class name of the provider entity to create
     * @return the relative path of the modification template
     */
    public String getModifyTemplate( String strProviderClassName )
    {
        IProviderService service = _services.get( strProviderClassName );

        return service.getModifyTemplate(  );
    }

    /**
     * Get the deletion jsp of a service
     * @param strProviderClassName The class name of the provider entity to create
     * @return the relative path of the deletion jsp
     */
    public String getDeleteJsp( String strProviderClassName )
    {
        IProviderService service = _services.get( strProviderClassName );

        return service.getDeleteJsp(  );
    }

    /**
     * Creates a provider
     * @param strProviderClassName The class name of the provider entity to create
     * @param provider The provider to create in database
     * @param request The HTTP request
     */
    public void doCreateProvider( String strProviderClassName, Provider provider, HttpServletRequest request )
    {
        IProviderService service = _services.get( strProviderClassName );
        service.doCreateProvider( provider, request );
    }

    /**
     * Modifies a provider.
     * @param strProviderClassName The class name of the provider entity to modify
     * @param provider The provider to modify in database
     * @param request The HTTP request
     */
    public void doModifyProvider( String strProviderClassName, Provider provider, HttpServletRequest request )
    {
        IProviderService service = _services.get( strProviderClassName );
        service.doModifyProvider( provider, request );
    }

    /**
     * Delete a provider
     * @param nIdProvider the identifier of the provider to delete
     * @param strProviderClassName The class name of the provider entity to delete
     */
    public void doDeleteProvider( String strProviderClassName, int nIdProvider )
    {
        IProviderService service = _services.get( strProviderClassName );
        service.doDeleteProvider( nIdProvider );
    }

    /**
     * Initaliaze the model in parameter for a creation
     * @param strProviderClassName The class name of the provider entity to delete
     * @param model the model which will be passed in the creation template
     */
    public void initModelForCreation( String strProviderClassName, Map<String, Object> model )
    {
        IProviderService service = _services.get( strProviderClassName );
        service.initModelForCreation( model );
    }

    /**
     * Initaliaze the model in parameter for a modification
     * @param strProviderClassName The class name of the provider entity to delete
     * @param nIdProvider the id of the privider to modify
     * @param model the model which will be passed in the modification template
     */
    public void initModelForModification( String strProviderClassName, int nIdProvider, Map<String, Object> model )
    {
        IProviderService service = _services.get( strProviderClassName );
        service.initModelForModification( nIdProvider, model );
    }

    /**
     * Get all the registered services
     * @return The list of all the registered services
     */
    public List<IProviderService> getAllRegisteredServices(  )
    {
        return new ArrayList<IProviderService>( _services.values(  ) );
    }

    /**
     * Get the {@link ObjectDefinition} objects of all the registered services
     * @return The {@link ObjectDefinition} objects of all the registered services
     */
    public List<ObjectDefinition> getAllRegisteredObjectDefinition(  )
    {
        Collection<IProviderService> listServices = _services.values(  );
        List<ObjectDefinition> listAllObjectDefinitions = new ArrayList<ObjectDefinition>(  );

        for ( IProviderService providerService : listServices )
        {
            listAllObjectDefinitions.add( providerService.getObjectDefinition(  ) );
        }

        return listAllObjectDefinitions;
    }

    /**
     * Get the {@link ObjectDefinition} objects of the registered services where the given user is authorized to create
     * @param user the user
     * @return The list of {@link ObjectDefinition} for which the given user is authorized to create a provider
     */
    public List<ObjectDefinition> getAuthorizedObjectDefinitionForCreation( AdminUser user )
    {
        Collection<IProviderService> listServices = _services.values(  );
        List<ObjectDefinition> listAllObjectDefinitions = new ArrayList<ObjectDefinition>(  );

        for ( IProviderService providerService : listServices )
        {
            if ( providerService.isAuthorizedToCreate( user ) )
            {
                listAllObjectDefinitions.add( providerService.getObjectDefinition(  ) );
            }
        }

        return listAllObjectDefinitions;
    }

    /**
     * Get the {@link ObjectDefinition} objects of the given provider entity
     * @param strProviderClassName The class name of the provider entity
     * @return the {@link ObjectDefinition} objects of the given class
     */
    public ObjectDefinition getObjectDefinition( String strProviderClassName )
    {
        IProviderService service = _services.get( strProviderClassName );

        return service.getObjectDefinition(  );
    }

    /**
     * Checks if a user is authorized to create the given provider entity
     * @param user the user
     * @param strProviderClassName The class name of the provider entity
     * @return true if the given user is authorized to create the given provider entity. False otherwise.
     */
    public boolean isAuthorizedToCreate( AdminUser user, String strProviderClassName )
    {
        IProviderService service = _services.get( strProviderClassName );

        return service.isAuthorizedToCreate( user );
    }

    /**
     * Checks if a user is authorized to modify the given provider entity
     * @param user the user
     * @param strProviderClassName The class name of the provider entity
     * @return true if the given user is authorized to modify the given provider entity. False otherwise.
     */
    public boolean isAuthorizedToModify( AdminUser user, String strProviderClassName )
    {
        IProviderService service = _services.get( strProviderClassName );

        return service.isAuthorizedToModify( user );
    }

    /**
     * Checks if a user is authorized to delete the given provider entity
     * @param user the user
     * @param strProviderClassName The class name of the provider entity
     * @return true if the given user is authorized to delete the given provider entity. False otherwise.
     */
    public boolean isAuthorizedToDelete( AdminUser user, String strProviderClassName )
    {
        IProviderService service = _services.get( strProviderClassName );

        return service.isAuthorizedToDelete( user );
    }
}
