/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Product;
import fr.paris.lutece.plugins.stock.business.ProductFilter;
import fr.paris.lutece.portal.business.user.AdminUser;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * IProductService
 *
 */
public interface IProductService
{
    /**
     * Return create template url for a product
     * @return create template url
     */
    String getCreateTemplate(  );

    /**
     * Return modify template url for a product
     * @return modify template url
     */
    String getModifyTemplate(  );

    /**
     * Return template url for ordering a product
     * @return template url
     */
    String getOrderTemplate(  );

    /**
     * Return the jsp for deleting a product
     * @return the jsp path
     */
    String getDeleteJsp(  );

    /**
     * Return the class of the product
     * @return the class
     */
    String getProductClass(  );

    /**
     * Init the product service and register it in the global product service
     */
    void init(  );

    /**
     * Create the product
     * @param product the product to create
     * @param request the request
     * @return error if occurs
     */
    String doCreateProduct( Product product, HttpServletRequest request );

    /**
     * Modify the product
     * @param product the product to modify
     * @param request the request
     * @return error if occurs
     */
    String doModifyProduct( Product product, HttpServletRequest request );

    /**
     * Delete the product
     * @param nIdProduct the id of the product to delete
     */
    void doDeleteProduct( int nIdProduct );

    /**
     * Insert the needed tag by the product for modification into the model
     * @param nIdProduct the product id
     * @param model the model
     * @param request The Http request
     */
    void initModelForModification( int nIdProduct, Map<String, Object> model, HttpServletRequest request );

    /**
     * Insert the needed tag by the product for ordering into the model
     * @param nIdProduct the product id
     * @param model the model
     * @param request The Http request
     */
    void initModelForOrder( int nIdProduct, Map<String, Object> model, HttpServletRequest request );

    /**
     * Return the product object
     * @param nIdProduct the product id
     * @return the product
     */
    Product getProduct( int nIdProduct );

    /**
     * Return a filtered list of product
     * @param filter the product filter
     * @return list of product
     */
    List<Product> findByFilter( ProductFilter filter );

    /**
     * Return the object definition
     * @return the object definition
     */
    ObjectDefinition getObjectDefinition(  );

    /**
     * Checks if if the given user is authorized to create a product
     * @param user The user who wants to create a product
     * @return True if the given user is authorized to create a product. False otherwise.
     */
    boolean isAuthorizedToCreate( AdminUser user );

    /**
     * Checks if if the given user is authorized to modify a product
     * @param user The user who wants to modify a product
     * @return True if the given user is authorized to modify a product. False otherwise.
     */
    boolean isAuthorizedToModify( AdminUser user );

    /**
     * Checks if if the given user is authorized to delete a product
     * @param user The user who wants to delete a product
     * @return True if the given user is authorized to delete a product. False otherwise.
     */
    boolean isAuthorizedToDelete( AdminUser user );

    /**
     * Get the message when a user is not authorized to create a product
     * @return The message
     */
    String getMessageNotAuthorizedForCreate(  );

    /**
     * Get the message when a user is not authorized to modify a product
     * @return The message
     */
    String getMessageNotAuthorizedForModification(  );

    /**
     * Get the message when a user is not authorized to delete a product
     * @return The message
     */
    String getMessageNotAuthorizedForDelete(  );

    /**
     * Update a product in the database
     * @param product the product to update in the database
     */
    void updateProduct( Product product );
}
